{% extends "base.html" %}
{% load cms_tags %}

{% block title %}{% page_attribute "page_title" %}{% endblock title %}

{% block content %}
    <div class="sidebar">
        {% placeholder "sidebar" %}
    </div>
    <div class="content">
        {% placeholder "content" %}
    </div>
{% endblock content %}
